/*tkmacicon: implements a simple Cocoa-based mechanism for retrieving file icons in a Tk application on OS X. (c) 2009-2014 WordTech Communications LLC. Partly based on code found at http://www.cocoadev.com/ThumbnailImages. License: standard Tcl license,  http://www.tcl.tk/software/tcltk/license.html.*/


#import <tcl.h>
#import <Cocoa/Cocoa.h>


@interface MacIcon:  NSObject

-(char*) makeIconFromFilePath: (NSString *) filePath imagewidth: (float ) width imageheight: (float )height;
-(char*) makeIconFromFileType: (NSString *) fileType imagewidth: (float ) width imageheight: (float )height;

@end

@implementation MacIcon

-(char*) makeIconFromFilePath: (NSString *) filePath imagewidth: (float) width imageheight: (float)height  {


  //retrieve the icon
  NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:filePath];

  //get bitmap representation of icon
  NSBitmapImageRep *resizeicon =  [[icon representations] objectAtIndex: 0];

  //initialize bits for conversion
  NSBitmapImageRep *output = nil;
  NSImage *scratch = nil;
  int w,h,nw,nh;
  NSData *bitmapData;
       
  // source image size
  w = nw = [resizeicon pixelsWide];
  h = nh = [resizeicon pixelsHigh];
    
  if (w>width || h>height) {
    float wr, hr;
	
    // ratios
    wr = w/(float)width;
    hr = h/(float)height;
	
	
    if (wr>hr)  { // landscape	{
      nw = width;
      nh = h/wr;
    }
    else { // portrait
      nh = height;
      nw = w/hr;
    };
	
  };
    
  // image to render into
  scratch = [[[NSImage alloc] initWithSize:NSMakeSize(nw, nh)] autorelease];
    
  // draw into image, to scale it
  [scratch lockFocus];
  [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
  [resizeicon drawInRect:NSMakeRect(0.0, 0.0, nw, nh)];
  output = [[[NSBitmapImageRep alloc] initWithFocusedViewRect:NSMakeRect(0,0,nw,nh)] autorelease];
  [scratch unlockFocus];
  bitmapData = [output representationUsingType:NSPNGFileType
				    properties:nil];

  NSString *base64String = [bitmapData base64EncodedStringWithOptions:0];

  char* imgdata = [base64String UTF8String];
 
  return imgdata;

}

-(char*) makeIconFromFileType: (NSString *) fileType imagewidth: (float) width imageheight: (float)height {

  //retrieve the icon
  NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFileType:fileType];

  //get bitmap representation of icon
  NSBitmapImageRep *resizeicon = [[icon representations] objectAtIndex: 0];

  //initialize bits for conversion
  NSBitmapImageRep *output = nil;
  NSImage *scratch = nil;
  int w,h,nw,nh;
  NSData *bitmapData;
       
  // source image size
  w = nw = [resizeicon pixelsWide];
  h = nh = [resizeicon pixelsHigh];
    
  if (w>width || h>height) {
    float wr, hr;
	
    // ratios
    wr = w/(float)width;
    hr = h/(float)height;
	
	
    if (wr>hr)  { // landscape	{
      nw = width;
      nh = h/wr;
    }
    else { // portrait
      nh = height;
      nw = w/hr;
    };
	
  };
    
  // image to render into
  scratch = [[[NSImage alloc] initWithSize:NSMakeSize(nw, nh)] autorelease];
    
  // draw into image, to scale it
  [scratch lockFocus];
  [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
  [resizeicon drawInRect:NSMakeRect(0.0, 0.0, nw, nh)];
  output = [[[NSBitmapImageRep alloc] initWithFocusedViewRect:NSMakeRect(0,0,nw,nh)] autorelease];
  [scratch unlockFocus];
  bitmapData = [output representationUsingType:NSPNGFileType
				    properties:nil];

  NSString *base64String = [bitmapData base64EncodedStringWithOptions:0];

  char* imgdata = [base64String UTF8String];
  return imgdata;

    
}


@end 

MacIcon *tkmacicon;


int GetMacIconFromPath (ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv[]);
int GetMacIconFromType (ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv[]);
int Tkmacicon_Init (Tcl_Interp *ip);
int Tkmacicon_SafeInit(Tcl_Interp *ip);


// Tcl command to get a file icon
int GetMacIconFromPath (ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv[]) {
	
  NSString *iconpath;
  NSString *iconfile;
  double iconwidth;
  double iconheight;
  double tclwidth;
  double tclheight;
 
  //cast double vars to float for Cocoa method args 
  float outputheight; 
  float outputwidth; 
	
  //set up memory pool
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		
  //need proper number of args	    
  if(objc != 4) {        
    Tcl_WrongNumArgs(ip, 1, objv, "path width height");        
    return TCL_ERROR;    
  }
    
  //initialize the instance of tkmacicon
  MacIcon *tkmacicon = [[MacIcon alloc ] init];

  //convert args to usable Cocoa objects
  iconpath = [NSString stringWithUTF8String:Tcl_GetString(objv[1])];
  iconwidth =  Tcl_GetDoubleFromObj(ip, objv[2], &tclwidth);
  iconheight = Tcl_GetDoubleFromObj(ip, objv[3], &tclheight);
  outputwidth = (float) tclwidth;
  outputheight = (float) tclheight;

        
  //retrieve the icon and write it to file
  char *result = [tkmacicon makeIconFromFilePath:iconpath imagewidth:outputwidth imageheight:outputheight];
  Tcl_AppendResult(ip, result, NULL);
       
  //release the memory 
  [pool release];   
  return TCL_OK;
}


// Tcl command to get a file icon
int GetMacIconFromType (ClientData cd, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv[]) {
	
  NSString *icontype;
  NSString *iconfile;
  double iconwidth;
  double iconheight;
  double tclwidth;
  double tclheight;
 
  //cast double vars to float for Cocoa method args 
  float outputheight; 
  float outputwidth; 
	
  //set up memory pool
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		
  //need proper number of args	    
  if(objc != 4) {        
    Tcl_WrongNumArgs(ip, 1, objv, "type width height");        
    return TCL_ERROR;    
  }
    
  //initialize the instance of tkmacicon
  MacIcon *tkmacicon = [[MacIcon alloc ] init];

  //convert args to usable Cocoa objects
  icontype = [NSString stringWithUTF8String:Tcl_GetString(objv[1])];
  iconwidth =  Tcl_GetDoubleFromObj(ip, objv[2], &tclwidth);
  iconheight = Tcl_GetDoubleFromObj(ip, objv[3], &tclheight);
  outputwidth = (float) tclwidth;
  outputheight = (float) tclheight;
      
  //retrieve the icon and write it to file
  char *result = [tkmacicon makeIconFromFileType:icontype imagewidth:outputwidth imageheight:outputheight];
  Tcl_AppendResult(ip, result, NULL);
           
  //release the memory 
  [pool release];   
  return TCL_OK;
}


//initalize the package in the tcl interpreter, create tcl commands
int Tkmacicon_Init (Tcl_Interp *ip) {
	
  //set up an autorelease pool
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
  if (Tcl_InitStubs(ip, "8.5", 0) == NULL) { 
    return TCL_ERROR;
  }

  Tcl_CreateObjCommand(ip, "::tkmacicon::geticonfrompath", GetMacIconFromPath,(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateObjCommand(ip, "::tkmacicon::geticonfromtype", GetMacIconFromType,(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	      

  if (Tcl_PkgProvide(ip, "tkmacicon", "1.2") != TCL_OK) {
    return TCL_ERROR;
  }

  //release memory
  [pool release];
	
  return TCL_OK;
	

}

int Tkmacicon_SafeInit(Tcl_Interp *ip) {
  return Tkmacicon_Init(ip);
}


